@@ -36,43 +36,48 @@ class API(object):
 
     def render_urls(self):
         """ Configure the API endpoints. """
-        self.__url_base = 'https://' + self.__hostname + '/rest_api/' + \
-                          self.__rest_version
-
+        self.__url_base = 'https://' + self.__hostname + '/rest_api/' + self.__rest_version
         self.__url_version = 'https://' + self.__hostname + '/rest_api/version'
-        self.__url_auth = self.__url_base + '/auth'
-        self.__url_login = self.__url_base + '/panel/login'
-        self.__url_status = self.__url_base + '/status'
-        self.__url_alarms = self.__url_base + '/alarms'
-        self.__url_alerts = self.__url_base + '/alerts'
-        self.__url_troubles = self.__url_base + '/troubles'
-        self.__url_panel_info = self.__url_base + '/panel_info'
-        self.__url_events = self.__url_base + '/events'
-        self.__url_devices = self.__url_base + '/devices'
-        self.__url_set_state = self.__url_base + '/set_state'
-        self.__url_locations = self.__url_base + '/locations'
-        self.__url_active_users_info = self.__url_base + '/users'
-        self.__url_process_status = self.__url_base + '/process_status?process_tokens='
 
-        # New Endpoints ([X] = Implemented, [-] = Skip)
-        self.__url_access_grant = self.__url_base + '/access/grant'                         # [X]
-        self.__url_access_revoke = self.__url_base + '/access/revoke'                       # [X]
-        self.__url_activate_siren = self.__url_base + '/activate_siren'                     # [X]
-        self.__url_apptype = self.__url_base + '/apptype'                                   # [-]
-        self.__url_cameras = self.__url_base + '/cameras'                                   # [X]
-        self.__url_disable_siren = self.__url_base + '/disable_siren'                       # [X]
-        self.__url_feature_set  = self.__url_base + '/feature_set'                          # [X]
-        self.__url_home_automation_devices = self.__url_base + '/home_automation_devices'   # [-]
-        self.__url_make_video = self.__url_base + '/make_video'                             # [-]
-        self.__url_notifications_email = self.__url_base + '/notifications/email'           # [ ]
-        self.__url_panels = self.__url_base + '/panels'                                     # [X]
-        self.__url_password_reset = self.__url_base + '/password/reset'                     # [X]
-        self.__url_password_reset_complete = self.__url_base + '/password/reset/complete'   # [X]
-        self.__url_set_name = self.__url_base + '/set_name'                                 # [X]
-        self.__url_set_user_code = self.__url_base + '/set_user_code'                       # [ ]
-        self.__url_smart_devices = self.__url_base + '/smart_devices'                       # [X]
-        self.__url_smart_devices_settings = self.__url_base + '/smart_devices/settings'     # [X]
-        self.__url_wakeup_sms = self.__url_base + '/wakeup_sms'                             # [X]
+        # API endpoints
+        self.__url_access_grant             = self.__url_base + '/access/grant'
+        self.__url_access_revoke            = self.__url_base + '/access/revoke'
+        self.__url_activate_siren           = self.__url_base + '/activate_siren'
+        self.__url_alarms                   = self.__url_base + '/alarms'
+        self.__url_alerts                   = self.__url_base + '/alerts'
+        self.__url_auth                     = self.__url_base + '/auth'
+        self.__url_cameras                  = self.__url_base + '/cameras'
+        self.__url_devices                  = self.__url_base + '/devices'
+        self.__url_disable_siren            = self.__url_base + '/disable_siren'
+        self.__url_events                   = self.__url_base + '/events'
+        self.__url_feature_set              = self.__url_base + '/feature_set'
+        self.__url_locations                = self.__url_base + '/locations'
+        self.__url_notifications_email      = self.__url_base + '/notifications/email'
+        self.__url_panel_login              = self.__url_base + '/panel/login'
+        self.__url_panel_info               = self.__url_base + '/panel_info'
+        self.__url_panels                   = self.__url_base + '/panels'
+        self.__url_password_reset           = self.__url_base + '/password/reset'
+        self.__url_password_reset_complete  = self.__url_base + '/password/reset/complete'
+        self.__url_process_status           = self.__url_base + '/process_status?process_tokens='
+        self.__url_set_name                 = self.__url_base + '/set_name'
+        self.__url_set_state                = self.__url_base + '/set_state'
+        self.__url_smart_devices            = self.__url_base + '/smart_devices'
+        self.__url_smart_devices_settings   = self.__url_base + '/smart_devices/settings'
+        self.__url_status                   = self.__url_base + '/status'
+        self.__url_troubles                 = self.__url_base + '/troubles'
+        self.__url_users                    = self.__url_base + '/users'
+        self.__url_wakeup_sms               = self.__url_base + '/wakeup_sms'
+
+        # To be implemented
+        self.__url_panel_add                = self.__url_base + '/panel/add'
+        self.__url_panel_rename             = self.__url_base + '/panel/rename'
+        self.__url_panel_unlink             = self.__url_base + '/panel/unlink'
+        self.__url_set_user_code            = self.__url_base + '/set_user_code'
+
+        # Will not be implemented
+        self.__url_apptype                  = self.__url_base + '/apptype'
+        self.__url_home_automation_devices  = self.__url_base + '/home_automation_devices'
+        self.__url_make_video               = self.__url_base + '/make_video'
 
     def set_rest_version(self, version):
         """ Set which version to use when connection to the API. """
@@ -115,6 +120,8 @@ class API(object):
                 if pair['value'] == 'wrong_combination':
                     if pair['key'] == 'email' or pair['key'] == 'password':
                         raise WrongUsernameOrPasswordError()
+                    if pair['key'] == 'panel_serial' or pair['key'] == 'master_user_code':
+                        raise WrongPanelSerialOrMasterUserCodeError()
         elif api['error'] == 10021: # WrongUserCode
             raise UserCodeIncorrectError()
         elif api['error'] == 400 and api['error_reason_code'] == 'PanelNotConnected':
@@ -249,26 +256,6 @@ class API(object):
         else:
             return False
 
-    def panel_login(self, panel_serial, user_code):
-        """ Try to login to the alarm panel and get a session token. """
-        login_info = {
-            'user_code': user_code,
-            'app_type': self.__app_type,
-            'app_id': self.__app_id,
-            'panel_serial': panel_serial
-        }
-
-        login_json = json.dumps(login_info, separators=(',', ':'))
-        res = self.__send_request(self.__url_login,
-                                       with_session_token=False,
-                                       data_json=login_json,
-                                       request_type='POST')
-        if res is not None:
-            self.__session_token = res['session_token']
-            return True
-        else:
-            return False
-
     def is_logged_in(self):
         """ Check if the session token is still valid. """
         try:
@@ -301,11 +288,6 @@ class API(object):
         siren_json = json.dumps(siren_data, separators=(',', ':'))
         return self.__send_request(self.__url_disable_siren, data_json=siren_json, request_type='POST')
 
-    def get_active_user_info(self):
-        """ Get information about the active users.
-        Note: Only master users can see the active_user_ids! """
-        return self.__send_request(self.__url_active_users_info, request_type='GET')
-
     def get_alarms(self):
         """ Get the current alarms. """
         return self.__send_request(self.__url_alarms, request_type='GET')
@@ -322,6 +304,10 @@ class API(object):
         """ Get all device specific information. """
         return self.__send_request(self.__url_devices, request_type='GET')
 
+    def get_email_notifications(self):
+        """ Get settings for the email notifications. """
+        return self.__send_request(self.__url_notifications_email, request_type='GET')
+
     def get_events(self):
         """ Get the alarm panel events. """
         return self.__send_request(self.__url_events, request_type='GET')
@@ -363,10 +349,65 @@ class API(object):
         """ Get the current troubles. """
         return self.__send_request(self.__url_troubles, request_type='GET')
 
+    def get_users(self):
+        """ Get information about the active users.
+        Note: Only master users can see the active_user_ids! """
+        return self.__send_request(self.__url_users, request_type='GET')
+
     def get_wakeup_sms(self):
         """ Get the settings needed to wake up the alarm panel via SMS. """
         return self.__send_request(self.__url_wakeup_sms, request_type='GET')
 
+    def panel_add(self, alias, panel_serial, access_proof, master_user_code):
+        """ Add a new alarm panel to the user account. A master user code is required. """
+        panel_data = {
+            'alias': alias, 
+            'panel_serial': panel_serial, 
+            'access_proof': access_proof, 
+            'master_user_code': master_user_code
+        }
+        panel_json = json.dumps(panel_data, separators=(',', ':'))
+        return self.__send_request(self.__url_panel_add, data_json=panel_json, request_type='POST')
+
+    def panel_login(self, panel_serial, user_code):
+        """ Try to login to the alarm panel and get a session token. """
+        login_info = {
+            'user_code': user_code,
+            'app_type': self.__app_type,
+            'app_id': self.__app_id,
+            'panel_serial': panel_serial
+        }
+
+        login_json = json.dumps(login_info, separators=(',', ':'))
+        res = self.__send_request(self.__url_panel_login,
+                                       with_session_token=False,
+                                       data_json=login_json,
+                                       request_type='POST')
+        if res is not None:
+            self.__session_token = res['session_token']
+            return True
+        else:
+            return False
+
+    def panel_rename(self, alias, panel_serial):
+        """ Rename an alarm panel. """
+        panel_data = {
+            'panel_serial': panel_serial, 
+            'alias': alias, 
+        }
+        panel_json = json.dumps(panel_data, separators=(',', ':'))
+        return self.__send_request(self.__url_panel_rename, data_json=panel_json, request_type='POST')
+
+    def panel_unlink(self, panel_serial, password, app_id):
+        """ Unlink an alarm panel from the user account. """
+        panel_data = {
+            'panel_serial': panel_serial, 
+            'password': password, 
+            'app_id': app_id, 
+        }
+        panel_json = json.dumps(panel_data, separators=(',', ':'))
+        return self.__send_request(self.__url_panel_unlink, data_json=panel_json, request_type='POST')
+
     def password_reset(self, email):
         """ Request a password reset email. An email will be sent to the email address provided. """
         reset_data = {'email': email}
@@ -379,12 +420,24 @@ class API(object):
         reset_json = json.dumps(reset_data, separators=(',', ':'))
         return self.__send_request(self.__url_password_reset_complete, data_json=reset_json, request_type='POST')
 
+    def set_email_notifications(self, mode):
+        """ Set settings for the email notifications. """
+        notification_data = {'mode': mode}
+        notification_json = json.dumps(notification_data, separators=(',', ':'))
+        return self.__send_request(self.__url_notifications_email, data_json=notification_json, request_type='POST')
+
     def set_name(self, object_class, id, name):
         """ Set the name of any type of object in the alarm system. """
         name_data = {'class': object_class, 'id': id, 'name': name}
         name_json = json.dumps(name_data, separators=(',', ':'))
         return self.__send_request(self.__url_set_name, data_json=name_json, request_type='POST')
 
+    def set_user_code(self, user_code, user_id):
+        """ Set the code of a user in the alarm system. """
+        code_data = {'user_code': user_code, 'user_id': user_id}
+        code_json = json.dumps(code_data, separators=(',', ':'))
+        return self.__send_request(self.__url_set_name, data_json=code_json, request_type='POST')
+
     def arm_home(self, partition):
         """ Arm in Home mode. """
         arm_info = {'partition': partition, 'state': 'HOME'}
